#!/usr/bin/env python

# draws a wireframe of an obj file onto active layer
# Author: Tin Tran 
# Date: 2014

import math
import string
from gimpfu import *
from array import array
def degtorad(angle):
	return (1.0 * angle / 180 * math.pi)
	
def is_number(s):
	try:
		float(s)
		return True
	except ValueError:
		return False

def read_vector(parts):
	vector = []
	for part in parts:
		if (is_number(part)):
			vector.append(float(part))
	return vector

def read_face(parts):
	face = []
	for part in parts:
		if (is_number(part)):
			face.append(int(part))
		else: #check and see if it's multiple values just read first value
			newparts = part.split('/')
			if (is_number(newparts[0])):
				face.append(int(newparts[0]))
			else:    #check and see if it starts with a 'v' which stands for vertex
				vparts = newparts[0].split('v')
				if (is_number(vparts[0])):
					face.append(int(vparts[0]))
	return face
	
def rotate_vectors(v, rotate_x, rotate_y, rotate_z):
	rotated_vectors = []       #return array
	radx = degtorad(rotate_x)
	rady = degtorad(rotate_y)
	radz = degtorad(rotate_z)
	
	for vec in v:
		rvec = [0,0,0]
	    #rotate X axis
		tmp = [vec[0],vec[1],vec[2]]
		rvec[0] = tmp[0]
		rvec[1] = math.cos(radx)*tmp[1] - math.sin(radx)*tmp[2]
		rvec[2] = math.sin(radx)*tmp[1] - math.cos(radx)*tmp[2]
		#rotate Y axis
		tmp = [rvec[0],rvec[1],rvec[2]]
		rvec[0] = math.cos(rady)*tmp[0] + math.sin(rady)*tmp[2]
		rvec[1] = tmp[1]
		rvec[2] = -math.sin(rady)*tmp[0] + math.cos(rady)*tmp[2]
		#rotate Z axis
		tmp = [rvec[0],rvec[1],rvec[2]]
		rvec[0] = math.cos(radz)*tmp[0] - math.sin(radz)*tmp[1]
		rvec[1] = math.sin(radz)*tmp[0] + math.cos(radz)*tmp[1]
		rvec[2] = tmp[2]
		#add it to rotated vectors
		rotated_vectors.append(rvec)
	return (rotated_vectors)
def project_2D(v,midx,midy,BZ,SCALE): #returns projected vectors,minZ,maxZ
	minZ =  32000
	maxZ = -32000
	projected_vectors = []
	for vec in v:
		tmp = [0,0,0]
		tmp[0] = midx + vec[0]*BZ/(vec[2]+BZ)*SCALE
		tmp[1] = midy + vec[1]*BZ/(vec[2]+BZ)*SCALE
		tmp[2] = vec[2]           #just copy z value for depth decisions later
		projected_vectors.append(tmp)  #add it to projected vectors
		minZ = min(minZ,tmp[2])
		maxZ = max(maxZ,tmp[2])
	return (projected_vectors,minZ,maxZ)	
		
def python_open_obj_file(image, layer, filename, rotatex, rotatey, rotatez, scale, bz) :
	vectors = []
	vectors.append([0,0,0]) #add an empty point so that faces can point to right indexes starting at 1
	pdb.gimp_message("length: " + str(len(vectors)))
	faces = []
	savedLines = []
	savedUniqueLines = []
	#create savedLines array of 256 elements holding empty arrays
	for j in range(0, 256):
		savedLines.append([])
		savedUniqueLines.append([])
	pdb.gimp_image_undo_group_start(image)
	
	image_width = pdb.gimp_image_width(image)
	image_height = pdb.gimp_image_height(image)
	pdb.gimp_message("Width:" + str(layer.width))
	pdb.gimp_message("Height:" + str(layer.height))
	gimp.progress_init("detecting white...")
	gimp.progress_update(1.0 * 100)
	pdb.gimp_progress_end()
	f = open(filename, 'r')
	while 1:
		line=f.readline()
		if not line: break
		parts = line.split(' ')
		if (parts[0] == "v"): #if it's vector read it
			vectors.append(read_vector(parts))
		if (parts[0] == "f"): #if it's a face read it
			faces.append(read_face(parts))
			
	f.close()
	
	rotated = rotate_vectors(vectors, rotatex, rotatey, rotatez)
	(projected,minZ,maxZ) = project_2D(rotated,image_width/2,image_height/2,bz,scale)
	rangeZ = maxZ - minZ
	
	#make face to end with starting point as well so it's wrapped fully around
	for i,face in enumerate(faces):
		face.append(face[0])
	
	#saves the user's brush to set it back later
	userbrushname = pdb.gimp_context_get_brush()
	#create our own brush
	brushname = pdb.gimp_brush_new("obj pen")
	brushradius = pdb.gimp_brush_set_radius(brushname,.5)
	brushshape = pdb.gimp_brush_set_shape(brushname,0)
	brushhardness = pdb.gimp_brush_set_hardness(brushname,1)
	pdb.gimp_context_set_brush(brushname)
	
	#sort out colorValue based on depth(z) and save it to savedLines (ordered)
	for face in faces:
		for j in range(0, (len(face)-1)):
			colorValue = int(255 - ((projected[face[j]][2] + projected[face[j+1]][2])/2-minZ)/rangeZ * 230)
			savedLines[colorValue].append([face[j],face[j+1]])
			
	#copy Uniquelines so we don't have to draw it multiple times
	for k in range(0,256):
		for line in savedLines[k]:
			if (([line[0],line[1]] not in savedUniqueLines[k]) and ([line[1],line[0]] not in savedUniqueLines[k])):
				savedUniqueLines[k].append(line)

				
	#draw it from darkest to brightest (previously calculated above)
	gimp.progress_init("drawing...")
	for j in range(0,256):
		gimp.progress_update(1.0 * j/255)
		pdb.gimp_context_set_foreground((j,j,j))
		for line in savedUniqueLines[j]:
			pdb.gimp_pencil(layer,4,[projected[line[0]][0],projected[line[0]][1],projected[line[1]][0],projected[line[1]][1]])
	pdb.gimp_progress_end()
	#delete brush when done
	pdb.gimp_brush_delete(brushname)
	pdb.gimp_context_set_brush(userbrushname) #set it back so we don't surprise user
		
	pdb.gimp_message("Completed!")
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
    "python_fu_open_obj_file",
    "Open an obj file",
    "Open obj file and draws it on active layer",
    "Tin Tran",
    "Tin Tran",
    "2014",
    "<Image>/File/Open Object file...",
    "RGB*, GRAY*",      # Create a new image, don't work on an existing one
    [
	(PF_FILE, "infilename", "File to open", "/Default/Path"),
	(PF_ADJUSTMENT, "rotatex", "Rotate X-axis", 270 ,(0,360,1)),
	(PF_ADJUSTMENT, "rotatey", "Rotate Y-axis", 0 ,(0,360,1)),
	(PF_ADJUSTMENT, "rotatez", "Rotate Z-axis", 0 ,(0,360,1)),
	(PF_ADJUSTMENT, "scale", "Scale", 160 ,(1,5000,.1)),
	(PF_ADJUSTMENT, "bz", "BZ distance", 600 ,(1,1000,10))
	],
    [],
    python_open_obj_file)

main()
